@@ -6,15 +6,17 @@
 
 #pragma once
 
+#include "columnview.h"
 #include <QCache>
+#include <QQmlPropertyMap>
 #include <QQuickItem>
 #include <QRandomGenerator>
-#include "columnview.h"
 
-static std::map<quint32,QVariant> s_knownVariants;
+static std::map<quint32, QVariant> s_knownVariants;
 class PageRouter;
 
-class ParsedRoute : public QObject {
+class ParsedRoute : public QObject
+{
     Q_OBJECT
 
 public:
@@ -22,11 +24,13 @@ public:
     QVariant data;
     QVariantMap properties;
     bool cache;
-    QQuickItem* item = nullptr;
-    void itemDestroyed() {
+    QQuickItem *item = nullptr;
+    void itemDestroyed()
+    {
         item = nullptr;
     }
-    QQuickItem* setItem(QQuickItem *newItem) {
+    QQuickItem *setItem(QQuickItem *newItem)
+    {
         auto ret = item;
         if (ret != nullptr) {
             disconnect(ret, &QObject::destroyed, this, &ParsedRoute::itemDestroyed);
@@ -37,16 +41,26 @@ public:
         }
         return ret;
     }
-    explicit ParsedRoute(const QString& name = QString(), QVariant data = QVariant(), QVariantMap properties = QVariantMap(), bool cache = false, QQuickItem *item = nullptr)
-        : name(name), data(data), properties(properties), cache(cache) {
+    explicit ParsedRoute(const QString &name = QString(),
+                         QVariant data = QVariant(),
+                         QVariantMap properties = QVariantMap(),
+                         bool cache = false,
+                         QQuickItem *item = nullptr)
+        : name(name)
+        , data(data)
+        , properties(properties)
+        , cache(cache)
+    {
         setItem(item);
     }
-    virtual ~ParsedRoute() {
+    virtual ~ParsedRoute()
+    {
         if (item) {
             item->deleteLater();
         }
     }
-    quint32 hash() {
+    quint32 hash()
+    {
         for (auto i = s_knownVariants.begin(); i != s_knownVariants.end(); i++) {
             if (i->second == data) {
                 return i->first;
@@ -59,38 +73,44 @@ public:
         s_knownVariants[number] = data;
         return number;
     }
-    bool equals(const ParsedRoute* rhs, bool countItem = false)
+    bool equals(const ParsedRoute *rhs, bool countItem = false)
     {
-        return name == rhs->name &&
-               data == rhs->data &&
-               (!countItem || item == rhs->item) &&
-               cache == rhs->cache;
+        /* clang-format off */
+        return name == rhs->name
+            && data == rhs->data
+            && (!countItem || item == rhs->item)
+            && cache == rhs->cache;
+        /* clang-format on */
     }
 };
 
 struct LRU {
     int size = 10;
-    QList<QPair<QString,quint32>> evictionList;
-    QMap<QPair<QString,quint32>,int> costs;
-    QMap<QPair<QString,quint32>,ParsedRoute*> items;
+    QList<QPair<QString, quint32>> evictionList;
+    QMap<QPair<QString, quint32>, int> costs;
+    QMap<QPair<QString, quint32>, ParsedRoute *> items;
 
-    ParsedRoute* take(QPair<QString,quint32> key) {
+    ParsedRoute *take(QPair<QString, quint32> key)
+    {
         auto ret = items.take(key);
         evictionList.removeAll(key);
         return ret;
     }
-    int totalCosts() {
+    int totalCosts()
+    {
         int ret = 0;
-        for (auto cost : costs) {
+        for (auto cost : qAsConst(costs)) {
             ret += cost;
         }
         return ret;
     }
-    void setSize(int size = 10) {
+    void setSize(int size = 10)
+    {
         this->size = size;
         prune();
     }
-    void prune() {
+    void prune()
+    {
         while (size < totalCosts()) {
             auto key = evictionList.last();
             auto item = items.take(key);
@@ -99,7 +119,8 @@ struct LRU {
             evictionList.takeLast();
         }
     }
-    void insert(QPair<QString,quint32> key, ParsedRoute *newItem, int cost) {
+    void insert(QPair<QString, quint32> key, ParsedRoute *newItem, int cost)
+    {
         if (items.contains(key)) {
             auto item = items.take(key);
             evictionList.removeAll(key);
@@ -118,7 +139,7 @@ class PageRouterAttached;
 
 /**
  * Item holding data about when to preload a route.
- * 
+ *
  * @code{.qml}
  * preload {
  *   route: "updates-page"
@@ -132,7 +153,7 @@ class PreloadRouteGroup : public QObject
 
     /**
      * @brief The route to preload.
-     * 
+     *
      * When the condition is false, the route will not be preloaded.
      */
     Q_PROPERTY(QJSValue route MEMBER m_route WRITE setRoute NOTIFY changed)
@@ -140,23 +161,26 @@ class PreloadRouteGroup : public QObject
 
     /**
      * @brief When the route should be preloaded.
-     * 
+     *
      * When the condition is false, the route will not be preloaded.
      */
     Q_PROPERTY(bool when MEMBER m_when NOTIFY changed)
     bool m_when;
 
     void handleChange();
-    PageRouterAttached* m_parent;
+    PageRouterAttached *m_parent;
 
 public:
     ~PreloadRouteGroup();
-    void setRoute(QJSValue route) {
+    void setRoute(QJSValue route)
+    {
         m_route = route;
         Q_EMIT changed();
     }
-    PreloadRouteGroup(QObject *parent) : QObject(parent) {
-        m_parent = qobject_cast<PageRouterAttached*>(parent);
+    PreloadRouteGroup(QObject *parent)
+        : QObject(parent)
+    {
+        m_parent = qobject_cast<PageRouterAttached *>(parent);
         Q_ASSERT(m_parent);
         connect(this, &PreloadRouteGroup::changed, this, &PreloadRouteGroup::handleChange);
     }
@@ -165,9 +189,9 @@ public:
 
 /**
  * Item representing a route the PageRouter can navigate to.
- * 
+ *
  * @include PageRoute.qml
- * 
+ *
  * @see PageRouter
  */
 class PageRoute : public QObject
@@ -176,7 +200,7 @@ class PageRoute : public QObject
 
     /**
      * @brief The name of this route.
-     * 
+     *
      * This name should be unique per PageRoute in a PageRouter.
      * When two PageRoutes have the same name, the one listed first
      * in the PageRouter will be used.
@@ -185,32 +209,32 @@ class PageRoute : public QObject
 
     /**
      * @brief The page component of this route.
-     * 
+     *
      * This should be an instance of Component with a Kirigami::Page inside
      * of it.
      */
-    Q_PROPERTY(QQmlComponent* component MEMBER m_component READ component)
+    Q_PROPERTY(QQmlComponent *component MEMBER m_component READ component)
 
     /**
      * @brief Whether pages generated by this route should be cached or not.
-     * 
+     *
      * This should be an instance of Component with a Kirigami::Page inside
      * of it.
-     * 
+     *
      * This will not work:
-     * 
+     *
      * @include PageRouterCachePagesDont.qml
-     * 
+     *
      * This will work:
-     * 
+     *
      * @include PageRouterCachePagesDo.qml
-     * 
+     *
      */
     Q_PROPERTY(bool cache MEMBER m_cache READ cache)
 
     /**
      * @brief How expensive this route is on memory.
-     * 
+     *
      * This affects caching, as the sum of costs of routes in the cache
      * can never exceed the cache's cap.
      */
@@ -224,73 +248,85 @@ Q_SIGNALS:
 
 private:
     QString m_name;
-    QQmlComponent* m_component;
+    QQmlComponent *m_component = nullptr;
     bool m_cache = false;
     int m_cost = 1;
 
 public:
-    QQmlComponent* component() { return m_component; };
-    QString name() { return m_name; };
-    bool cache() { return m_cache; };
-    int cost() { return m_cost; };
+    QQmlComponent *component()
+    {
+        return m_component;
+    };
+    QString name()
+    {
+        return m_name;
+    };
+    bool cache()
+    {
+        return m_cache;
+    };
+    int cost()
+    {
+        return m_cost;
+    };
 };
 
 /**
  * An item managing pages and data of a ColumnView using named routes.
- * 
+ *
  * <br> <br>
- * 
+ *
  * ## Using a PageRouter
- * 
+ *
  * Applications typically manage their contents via elements called "pages" or "screens."
  * In Kirigami, these are called @link org::kde::kirigami::Page  Pages @endlink and are
  * arranged in @link PageRoute routes @endlink using a PageRouter to manage them. The PageRouter
  * manages a stack of @link org::kde::kirigami::Page Pages @endlink created from a pool of potential
  * @link PageRoute PageRoutes @endlink.
- * 
+ *
  * Unlike most traditional stacks, a PageRouter provides functions for random access to its pages
  * with navigateToRoute and routeActive.
- * 
+ *
  * When your user interface fits the stack paradigm and is likely to use random access navigation,
  * using the PageRouter is appropriate. For simpler navigation, it is more appropriate to avoid
  * the overhead of a PageRouter by using a @link org::kde::kirigami::PageRow PageRow  @endlink
  * instead.
- * 
+ *
  * <br> <br>
- * 
+ *
  * ## Navigation Model
- * 
+ *
  * A PageRouter draws from a pool of @link PageRoute PageRoutes @endlink in order to construct
  * its stack.
- * 
+ *
  * @image html PageRouterModel.svg width=50%
- * 
+ *
  * <br> <br>
- * 
+ *
  * You can push pages onto this stack...
- * 
+ *
  * @image html PageRouterPush.svg width=50%
- * 
+ *
  * ...or pop them off...
- * 
+ *
  * @image html PageRouterPop.svg width=50%
- * 
+ *
  * ...or navigate to an arbitrary collection of pages.
- * 
+ *
  * @image html PageRouterNavigate.svg width=50%
- * 
+ *
  * <br> <br>
- * 
+ *
  * Components are able to query the PageRouter about the currently active routes
  * on the stack. This is useful for e.g. a card indicating that the page it takes
  * the user to is currently active.
- * 
+ *
  * <br> <br>
- * 
+ *
  * ## Example
- * 
+ *
  * @include PageRouter.qml
- * 
+ *
  * @see PageRouterAttached
  * @see PageRoute
  */
@@ -301,7 +337,7 @@ class PageRouter : public QObject, public QQmlParserStatus
 
     /**
      * @brief The named routes a PageRouter can navigate to.
-     * 
+     *
      * @include PageRouterRoutes.qml
      */
     Q_PROPERTY(QQmlListProperty<PageRoute> routes READ routes)
@@ -310,62 +346,85 @@ class PageRouter : public QObject, public QQmlParserStatus
 
     /**
      * @brief The initial route.
-     * 
+     *
      * `initialRoute` is the page that the PageRouter will push upon
      * creation. Changing it after creation will cause the PageRouter to reset
      * its state. Not providing an `initialRoute` will result in undefined
      * behavior.
-     * 
+     *
      * @include PageRouterInitialRoute.qml
      */
     Q_PROPERTY(QJSValue initialRoute READ initialRoute WRITE setInitialRoute NOTIFY initialRouteChanged)
 
     /**
      * @brief The ColumnView being puppeted by the PageRouter.
-     * 
+     *
      * All PageRouters should be created with a ColumnView, and creating one without
      * a ColumnView is undefined behaviour.
-     * 
+     *
      * @warning You should **not** directly interact with a ColumnView being puppeted
      * by a PageRouter. Instead, use a PageRouter's functions to manipulate the
      * ColumnView.
-     * 
+     *
      * @include PageRouterColumnView.qml
      */
-    Q_PROPERTY(ColumnView* pageStack MEMBER m_pageStack NOTIFY pageStackChanged)
+    Q_PROPERTY(ColumnView *pageStack MEMBER m_pageStack NOTIFY pageStackChanged)
 
     /**
      * @brief How large the cache can be.
-     * 
+     *
      * The combined costs of cached routes will never exceed the cache capacity.
      */
     Q_PROPERTY(int cacheCapacity READ cacheCapacity WRITE setCacheCapacity)
 
     /**
      * @brief How large the preloaded pool can be.
-     * 
+     *
      * The combined costs of preloaded routes will never exceed the pool capacity.
      */
     Q_PROPERTY(int preloadedPoolCapacity READ preloadedPoolCapacity WRITE setPreloadedPoolCapacity)
 
+    /**
+     * Exposes the data of all pages on the stack, preferring pages on the top
+     * (e.g. most recently pushed) to pages pushed on the bottom (least recently
+     * pushed).
+     */
+    Q_PROPERTY(QQmlPropertyMap *params READ params CONSTANT)
+
 private:
+    /**
+     * The map exposing to QML all the params of the stack. This is a
+     * QQmlPropertyMap to allow binding to param values. This *does* lack
+     * the ability to drop items, but the amount of all params in an app
+     * is overwhelmingly likely to be fixed, not dynamic.
+     */
+    QSharedPointer<QQmlPropertyMap> m_paramMap;
+
+    /**
+     * Reevaluate the properties of the param map by going through all of the
+     * routes on the stack to determine the topmost value for every parametre.
+     *
+     * Should be called for every time a route is pushed, popped, or modified.
+     */
+    void reevaluateParamMapProperties();
+
     /**
      * @brief The routes the PageRouter is aware of.
-     * 
+     *
      * Generally, this should not be mutated from C++, only read.
      */
-    QList<PageRoute*> m_routes;
+    QList<PageRoute *> m_routes;
 
     /**
      * @brief The PageRouter being puppeted.
-     * 
+     *
      * m_pageRow is the ColumnView this PageRouter puppets.
      */
-    ColumnView* m_pageStack = nullptr;
+    ColumnView *m_pageStack = nullptr;
 
     /**
      * @brief The route that the PageRouter will load on completion.
-     * 
+     *
      * m_initialRoute is the raw QJSValue from QML that will be
      * parsed into a ParsedRoute struct on construction.
      * Generally, this should not be mutated from C++, only read.
@@ -374,22 +433,22 @@ private:
 
     /**
      * @brief The current routes pushed on the PageRow.
-     * 
+     *
      * Generally, the state of m_pageRow and m_currentRoutes
      * should be kept in sync. Undesirable behaviour will result
      * from desynchronisation of the two.
      */
-    QList<ParsedRoute*> m_currentRoutes;
+    QList<ParsedRoute *> m_currentRoutes;
 
     /**
      * @brief Cached routes.
-     * 
+     *
      * An LRU cache of ParsedRoutes with items that were previously on the stack.
      */
     LRU m_cache;
 
     /** @brief Preloaded routes.
-     * 
+     *
      * A LRU cache of ParsedRoutes with items that may be on the stack in the future,
      * but were not on the stack before.
      */
@@ -397,55 +456,55 @@ private:
 
     /**
      * @brief Helper function to push a route.
-     * 
+     *
      * This function has the shared logic between
      * navigateToRoute and pushRoute.
      */
-    void push(ParsedRoute* route);
+    void push(ParsedRoute *route);
 
     /**
      * @brief Helper function to access whether m_routes has a key.
-     * 
+     *
      * This function abstracts the QJSValue.
      */
-    bool routesContainsKey(const QString &key);
+    bool routesContainsKey(const QString &key) const;
 
     /**
      * @brief Helper function to access the component of a key for m_routes.
-     * 
+     *
      * The return value will be a nullptr if @p key does not exist in
      * m_routes.
      */
-    QQmlComponent *routesValueForKey(const QString &key);
+    QQmlComponent *routesValueForKey(const QString &key) const;
 
     /**
      * @brief Helper function to access the cache status of a key for m_routes.
-     * 
+     *
      * The return value will be false if @p key does not exist in
      * m_routes.
      */
-    bool routesCacheForKey(const QString &key);
+    bool routesCacheForKey(const QString &key) const;
 
     /**
      * @brief Helper function to access the cost of a key for m_routes.
-     * 
+     *
      * The return value will be -1 if @p key does not exist in
      * m_routes.
      */
-    int routesCostForKey(const QString &key);
+    int routesCostForKey(const QString &key) const;
 
     void preload(ParsedRoute *route);
     void unpreload(ParsedRoute *route);
 
     void placeInCache(ParsedRoute *route);
 
-    static void appendRoute(QQmlListProperty<PageRoute>* list, PageRoute*);
-    static int routeCount(QQmlListProperty<PageRoute>* list);
-    static PageRoute* route(QQmlListProperty<PageRoute>* list, int);
-    static void clearRoutes(QQmlListProperty<PageRoute>* list);
+    static void appendRoute(QQmlListProperty<PageRoute> *list, PageRoute *);
+    static int routeCount(QQmlListProperty<PageRoute> *list);
+    static PageRoute *route(QQmlListProperty<PageRoute> *list, int);
+    static void clearRoutes(QQmlListProperty<PageRoute> *list);
 
-    QVariant dataFor(QObject* object);
-    bool isActive(QObject* object);
+    QVariant dataFor(QObject *object);
+    bool isActive(QObject *object);
     void pushFromObject(QObject *object, QJSValue route, bool replace = false);
 
     friend class PageRouterAttached;
@@ -462,21 +521,38 @@ public:
 
     QQmlListProperty<PageRoute> routes();
 
+    QQmlPropertyMap *params()
+    {
+        return m_paramMap.data();
+    }
+
     QJSValue initialRoute() const;
     void setInitialRoute(QJSValue initialRoute);
 
-    int cacheCapacity() const { return m_cache.size; };
-    void setCacheCapacity(int size) { m_cache.setSize(size); };
+    int cacheCapacity() const
+    {
+        return m_cache.size;
+    };
+    void setCacheCapacity(int size)
+    {
+        m_cache.setSize(size);
+    };
 
-    int preloadedPoolCapacity() const { return m_preload.size; };
-    void setPreloadedPoolCapacity(int size) { m_preload.setSize(size); };
+    int preloadedPoolCapacity() const
+    {
+        return m_preload.size;
+    };
+    void setPreloadedPoolCapacity(int size)
+    {
+        m_preload.setSize(size);
+    };
 
     /**
      * @brief Navigate to the given route.
-     * 
+     *
      * Calling `navigateToRoute` causes the PageRouter to replace currently
      * active pages with the new route.
-     * 
+     *
      * @param route The given route for the PageRouter to navigate to.
      * A route is an array of variants or a single item. A string item will be interpreted
      * as a page without associated data. An object item will be interpreted
@@ -489,7 +565,7 @@ public:
      * @endcode
      * Navigating to a route not defined in a PageRouter's routes is undefined
      * behavior.
-     * 
+     *
      * @code{.qml}
      * Button {
      *     text: "Login"
@@ -503,20 +579,20 @@ public:
 
     /**
      * @brief Check whether the current route is on the stack.
-     * 
+     *
      * `routeActive` will return true if the given route
      * is on the stack.
-     * 
+     *
      * @param route The given route to check for.
-     * 
+     *
      * `routeActive` returns true for partial routes like
      * the following:
-     * 
+     *
      * @code{.js}
      * Kirigami.PageRouter.navigateToRoute(["/home", "/login", "/google"])
      * Kirigami.PageRouter.routeActive(["/home", "/login"]) // returns true
      * @endcode
-     * 
+     *
      * This only works from the root page, e.g. the following will return false:
      * @code{.js}
      * Kirigami.PageRouter.navigateToRoute(["/home", "/login", "/google"])
@@ -527,13 +603,13 @@ public:
 
     /**
      * @brief Appends a route to the currently navigated route.
-     * 
+     *
      * Calling `pushRoute` will append the given @p route to the currently navigated
      * routes. See navigateToRoute() if you want to replace the items currently on
      * the PageRouter.
-     * 
+     *
      * @param route The given route to push.
-     * 
+     *
      * @code{.js}
      * Kirigami.PageRouter.navigateToRoute(["/home", "/login"])
      * // The PageRouter is navigated to /home/login
@@ -545,10 +621,10 @@ public:
 
     /**
      * @brief Pops the last page on the router.
-     * 
+     *
      * Calling `popRoute` will result in the last page on the router getting popped.
      * You should not call this function when there is only one page on the router.
-     * 
+     *
      * @code{.js}
      * Kirigami.PageRouter.navigateToRoute(["/home", "/login"])
      * // The PageRouter is navigated to /home/login
@@ -560,23 +636,23 @@ public:
 
     /**
      * @brief Shifts keyboard focus and view to a given index on the PageRouter's stack.
-     * 
+     *
      * @param view The view to bring to focus. If this is an integer index, the PageRouter will
      * navigate to the given index. If it's a route specifier, the PageRouter will navigate
      * to the first route matching it.
-     * 
+     *
      * Navigating to route by index:
      * @code{.js}
      * Kirigami.PageRouter.navigateToRoute(["/home", "/browse", "/apps", "/login"])
      * Kirigami.PageRouter.bringToView(1)
      * @endcode
-     * 
+     *
      * Navigating to route by name:
      * @code{.js}
      * Kirigami.PageRouter.navigateToRoute(["/home", "/browse", "/apps", "/login"])
      * Kirigami.PageRouter.bringToView("/browse")
      * @endcode
-     * 
+     *
      * Navigating to route by data:
      * @code{.js}
      * Kirigami.PageRouter.navigateToRoute([{"route": "/page", "data": "red"},
@@ -590,7 +666,7 @@ public:
 
     /**
      * @brief Returns a QJSValue corresponding to the current pages on the stack.
-     * 
+     *
      * The returned value is in the same form as the input to navigateToRoute.
      */
     Q_INVOKABLE QJSValue currentRoutes() const;
@@ -608,7 +684,7 @@ Q_SIGNALS:
 /**
  * Attached object allowing children of a PageRouter to access its functions
  * without requiring the children to have the parent PageRouter's id.
- * 
+ *
  * @see PageRouter
  */
 class PageRouterAttached : public QObject
@@ -627,10 +703,10 @@ class PageRouterAttached : public QObject
      * Accessing this property outside of a PageRouter will result in undefined behaviour.
      */
     Q_PROPERTY(bool isCurrent READ isCurrent NOTIFY isCurrentChanged)
-    
+
     /**
      * Which route this PageRouterAttached should watch for.
-     * 
+     *
      * @include PageRouterWatchedRoute.qml
      */
     Q_PROPERTY(QJSValue watchedRoute READ watchedRoute WRITE setWatchedRoute NOTIFY watchedRouteChanged)
@@ -638,7 +714,7 @@ class PageRouterAttached : public QObject
     /**
      * Route preloading settings.
      */
-    Q_PROPERTY(PreloadRouteGroup* preload READ preload)
+    Q_PROPERTY(PreloadRouteGroup *preload READ preload)
 
     /**
      * Whether the watchedRoute is currently active.
@@ -649,7 +725,7 @@ private:
     explicit PageRouterAttached(QObject *parent = nullptr);
 
     QPointer<PageRouter> m_router;
-    PreloadRouteGroup* m_preload;
+    PreloadRouteGroup *m_preload;
     QVariant m_data;
     QJSValue m_watchedRoute;
 
@@ -660,11 +736,18 @@ private:
     friend class ParsedRoute;
 
 public:
-    PreloadRouteGroup* preload() const { return m_preload; };
-    PageRouter* router() const { return m_router; };
-    void setRouter(PageRouter* router) {
-    	m_router = router;
-    	Q_EMIT routerChanged();
+    PreloadRouteGroup *preload() const
+    {
+        return m_preload;
+    };
+    PageRouter *router() const
+    {
+        return m_router;
+    };
+    void setRouter(PageRouter *router)
+    {
+        m_router = router;
+        Q_EMIT routerChanged();
     }
     QVariant data() const;
     bool isCurrent() const;
@@ -680,10 +763,10 @@ public:
     Q_INVOKABLE void bringToView(QJSValue route);
     /**
      * @brief Push a route from this route on the stack.
-     * 
+     *
      * Replace the routes after the route this is invoked on
      * with the provided @p route.
-     * 
+     *
      * For example, if you invoke this method on the second route
      * in the PageRouter's stack, routes after the second
      * route will be replaced with the provided routes.
@@ -691,10 +774,10 @@ public:
     Q_INVOKABLE void pushFromHere(QJSValue route);
     /**
      * @brief Pop routes after this route on the stack.
-     * 
+     *
      * Pop the routes after the route this is invoked on with
      * the provided @p route.
-     * 
+     *
      * For example, if you invoke this method on the second route
      * in the PageRouter's stack, routes after the second route
      * will be removed from the stack.
@@ -702,7 +785,7 @@ public:
     Q_INVOKABLE void popFromHere();
     /**
      * @brief Replaces this route with the given routes on the stack.
-     * 
+     *
      * Behaves like pushFromHere, except the current route is also
      * popped.
      */
